from media_platform.auth.token import Token

_VERB = 'urn:service:image.operations'


class Gravity(object):
    center = 'center'
    north = "north"
    north_west = "north-west"
    west = "west"
    south_west = "south-west"
    south = "south"
    south_east = "south-east"
    east = "east"
    north_east = "north-east"


class Watermark(object):
    def __init__(self, path, opacity=50, proportions=0.25, gravity=Gravity.center):
        # type: (str, int, float, Gravity) -> None
        super(Watermark, self).__init__()
        self.path = path
        self.opacity = opacity
        self.proportions = proportions
        self.gravity = gravity

    def to_claim(self):
        # type: () -> dict
        watermark = {}
        if self.path:
            watermark['path'] = self.path
        if self.opacity:
            watermark['opacity'] = self.opacity
        if self.proportions:
            watermark['proportions'] = self.proportions
        if self.gravity:
            watermark['gravity'] = self.gravity

        return {
            'wmk': watermark
        }


class Policy(object):
    def __init__(self, path, max_width=None, max_height=None):
        # type: (str, int, int) -> None
        super(Policy, self).__init__()
        self.path = path
        self.max_width = max_width
        self.max_height = max_height

    def to_claim(self):
        # type: () -> dict
        policy = {}
        if self.path:
            policy['path'] = self.path
        if self.max_height:
            policy['height'] = '<=%s' % self.max_height
        if self.max_width:
            policy['width'] = '<=%s' % self.max_width

        return {
            'obj': [[policy]]
        }


class ImageToken(Token):
    def __init__(self, issuer, subject, policy=None, watermark=None, issued_at=None, expiration=None,
                 additional_claims=None, token_id=None):
        super(ImageToken, self).__init__(issuer, subject, [_VERB], issued_at, expiration, additional_claims, token_id)
        self.policy = policy
        self.watermark = watermark

    def to_claims(self):
        # type: () -> dict
        claims = super(ImageToken, self).to_claims()

        if self.watermark:
            claims.update(self.watermark.to_claim())
        if self.policy:
            claims.update(self.policy.to_claim())

        return claims

    @classmethod
    def from_token(cls, token):
        # type: (Token) -> ImageToken
        return cls(token.issuer, token.subject, None, None, token.issued_at, token.expiration, token.additional_claims,
                   token.id)



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.metadata.file_metadata import FileMetadata
# noinspection PyProtectedMember
from media_platform.metadata.file_metadata_deserializer import _FileMetadataDeserializer
from media_platform.service.media_platform_request import MediaPlatformRequest


class FileMetadataRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(FileMetadataRequest, self).__init__(authenticated_http_client, 'GET', base_url + '/files/metadata',
                                                  _FileMetadataDeserializer)

        self.path = None

    def set_path(self, path):
        # type: (str) -> FileMetadataRequest
        self.path = path
        return self

    def execute(self):
        # type: () -> FileMetadata
        return super(FileMetadataRequest, self).execute()

    def _params(self):
        # type: () -> dict
        return {
            'path': self.path
        }



from typing import List

from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.metadata.file_metadata import FileMetadata
# noinspection PyProtectedMember
from media_platform.metadata.file_metadata_deserializer import _FileMetadataDeserializer
from media_platform.service.media_platform_request import MediaPlatformRequest


class Detection(object):
    transparency = 'transparency'
    valid_values = [transparency]

    @classmethod
    def validate(cls, value):
        if value not in cls.valid_values:
            raise ValueError('Must be one of: ' + ','.join(cls.valid_values))

class ExtractMetadataRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(ExtractMetadataRequest, self).__init__(authenticated_http_client, 'GET',
                                                     base_url + '/files/metadata/extract', _FileMetadataDeserializer)

        self.path = None
        self.detections = []

    def set_path(self, path):
        # type: (str) -> ExtractMetadataRequest
        self.path = path
        return self

    def set_detections(self, detections):
        # type: (List[Detection]) -> ExtractMetadataRequest
        self.detections = detections
        return self

    def add_detection(self, detection):
        # type: (Detection) -> ExtractMetadataRequest
        self.detections.append(detection)
        return self

    def execute(self):
        # type: () -> FileMetadata
        return super(ExtractMetadataRequest, self).execute()

    def _params(self):
        # type: () -> dict
        return {
            'path': self.path,
            'detections': ','.join(self.detections)
        }



import os
import requests
import urllib3
from future.utils import PY3
from requests.adapters import HTTPAdapter
from requests.exceptions import RetryError
from requests.structures import CaseInsensitiveDict
from requests_toolbelt import MultipartEncoder
from requests_toolbelt.adapters.appengine import AppEngineAdapter
from typing import Type

from media_platform.lang.serialization import Deserializable
from media_platform.auth.app_authenticator import AppAuthenticator
from media_platform.exception.media_platform_exception import MediaPlatformException
from media_platform.http.response_processor import ResponseProcessor


class AuthenticatedHTTPClient(object):
    USER_AGENT = 'WixMP Python SDK 1.x'
    APPLICATION_JSON = 'application/json'
    RETRYABLE_CODES = [500, 503, 504, 429]
    RETRYABLE_METHODS = ['GET', 'POST', 'PUT', 'DELETE']
    TIMEOUT = 60

    def __init__(self, app_authenticator, retry_count=5, retry_backoff_factor=0.2):
        # type: (AppAuthenticator, int, float) -> None

        self._app_authenticator = app_authenticator
        self._session = requests.Session()

        retry = urllib3.Retry(total=retry_count, backoff_factor=retry_backoff_factor,
                              status_forcelist=self.RETRYABLE_CODES, method_whitelist=self.RETRYABLE_METHODS)

        self._session.mount('http://', self._matched_adapter(retry))
        self._session.mount('https://', self._matched_adapter(retry))

    def _matched_adapter(self, retry):
        server_software = os.environ.get('SERVER_SOFTWARE', '')

        if not PY3 and server_software.startswith('Google App Engine/'):
            return AppEngineAdapter(max_retries=retry)

        return HTTPAdapter(max_retries=retry)

    def get(self, url, params=None, payload_type=None):
        # type: (str, dict, Type[Deserializable]) -> Deserializable or None

        return self._send_request('GET', url, params=params, payload_type=payload_type)

    def post(self, url, data=None, payload_type=None):
        # type: (str, dict, Type[Deserializable]) -> Deserializable or None

        return self._send_request('POST', url, json=data, payload_type=payload_type)

    def put(self, url, data=None, payload_type=None):
        # type: (str, dict, Type[Deserializable]) -> Deserializable or None

        return self._send_request('PUT', url, json=data, payload_type=payload_type)

    def delete(self, url, params=None, payload_type=None):
        # type: (str, dict, Type[Deserializable]) -> Deserializable or None

        return self._send_request('DELETE', url, params=params, payload_type=payload_type)

    def post_data(self, url, content, mime_type, params=None, payload_type=None, filename=None,
                  response_processor=None):
        # type: (str, str, str, dict, Type[Deserializable], str, callable or None) -> Deserializable or None

        fields = {
            'file': (filename or 'file-name', content, mime_type)
        }
        fields.update(params)

        encoder = MultipartEncoder(fields)

        try:
            headers = self._base_headers()
            headers['Content-Type'] = encoder.content_type
            response = self._session.post(url, data=encoder, headers=headers)
        except RetryError as e:
            raise MediaPlatformException(cause=e)

        if response_processor:
            return response_processor(response)
        else:
            return ResponseProcessor.process(response, payload_type)

    def _send_request(self, verb, url, json=None, params=None, payload_type=None):
        # type: (str, str, dict, dict, Type[Deserializable]) -> Deserializable or None

        try:
            response = self._session.request(verb, url, params=params, json=json, headers=self._headers(), timeout=self.TIMEOUT)
        except RetryError as e:
            raise MediaPlatformException(cause=e)

        return ResponseProcessor.process(response, payload_type)

    def _headers(self):
        # type: () -> CaseInsensitiveDict

        headers = self._base_headers()

        signed_token = self._app_authenticator.default_signed_token()
        headers['Authorization'] = signed_token

        return headers

    def _base_headers(self):
        # type: () -> CaseInsensitiveDict

        headers = requests.utils.default_headers()
        headers['User-Agent'] = self.USER_AGENT
        headers['Accept'] = self.APPLICATION_JSON

        return headers



from media_platform.lang.serialization import Serializable


class ExternalAuthorization(Serializable):
    def __init__(self, headers):
        # type: (dict) -> None
        super(ExternalAuthorization, self).__init__()

        self.headers = headers

    def serialize(self):
        return {
            'headers': self.headers
        }

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ExternalAuthorization
        return cls(data['headers'])